@@ -1,7 +1,7 @@
 # descriptors.py
 #
 # This file is part of scqubits: a Python package for superconducting qubits,
-# arXiv:2107.08552 (2021). https://arxiv.org/abs/2107.08552
+# Quantum 5, 583 (2021). https://quantum-journal.org/papers/q-2021-11-17-583/
 #
 #    Copyright (c) 2019 and later, Jens Koch and Peter Groszkowski
 #    All rights reserved.
@@ -12,7 +12,7 @@
 
 # Recap on descriptors: see https://realpython.com/python-descriptors/
 
-from typing import Any, Generic, Type, TypeVar, Union
+from typing import Any, Generic, Type, TypeVar
 
 from scqubits.core.central_dispatch import DispatchClient
 
@@ -28,7 +28,7 @@ class ReadOnlyProperty(Generic[TargetType]):
         super().__init__()
 
     def __set_name__(self, owner, name: str):
-        self.name = "_" + name
+        self.name = f"_{name}"
 
     def __get__(self, instance: Any, *args, **kwargs) -> TargetType:
         if instance is None:  # when accessed on class level rather than instance level
@@ -47,7 +47,7 @@ class WatchedProperty(Generic[TargetType]):
 
     Parameters
     ----------
-    type:
+    target_type:
         type of watched property
     event:
         name of event to be triggered when property is changed
