@@ -1,6 +1,7 @@
 # flux_qubit.py
 #
-# This file is part of scqubits.
+# This file is part of scqubits: a Python package for superconducting qubits,
+# Quantum 5, 583 (2021). https://quantum-journal.org/papers/q-2021-11-17-583/
 #
 #    Copyright (c) 2019 and later, Jens Koch and Peter Groszkowski
 #    All rights reserved.
@@ -32,9 +33,8 @@ import scqubits.utils.spectrum_utils as spec_utils
 
 from scqubits.core.noise import NOISE_PARAMS, NoisySystem
 
+
 # -Flux qubit noise class
-
-
 class NoisyFluxQubit(NoisySystem, ABC):
     @abstractmethod
     def d_hamiltonian_d_EJ1(self) -> ndarray:
@@ -50,7 +50,7 @@ class NoisyFluxQubit(NoisySystem, ABC):
 
     @classmethod
     @abstractmethod
-    def supported_noise_channels(self) -> List[str]:
+    def supported_noise_channels(cls) -> List[str]:
         pass
 
     def tphi_1_over_f_cc1(
@@ -203,13 +203,14 @@ class NoisyFluxQubit(NoisySystem, ABC):
         get_rate: bool = False,
         **kwargs
     ) -> float:
-        r"""Calculate the 1/f dephasing time (or rate) due to critical current noise
+        r"""
+        Calculate the 1/f dephasing time (or rate) due to critical-current noise
         from all three Josephson junctions :math:`EJ1`, :math:`EJ2` and :math:`EJ3`.
         The combined noise is calculated by summing the rates from the individual
         contributions.
 
         Parameters
-        -----------
+        ----------
         A_noise:
             noise strength
         i:
@@ -221,8 +222,10 @@ class NoisyFluxQubit(NoisySystem, ABC):
         get_rate:
             get rate or time
 
-        Returns ------- decoherence time in units of :math:`2\pi ({\rm system\,\,
-        units})`, or rate in inverse units.
+        Returns
+        -------
+            decoherence time in units of :math:`2\pi` (system units),
+            or rate in inverse units.
         """
         if "tphi_1_over_f_cc" not in self.supported_noise_channels():
             raise RuntimeError(
@@ -299,18 +302,18 @@ class FluxQubit(base.QubitBaseClass, serializers.Serializable, NoisyFluxQubit):
         and `ParameterSweep`. If not provided, an id is auto-generated.
     """
 
-    EJ1 = descriptors.WatchedProperty("QUANTUMSYSTEM_UPDATE")
-    EJ2 = descriptors.WatchedProperty("QUANTUMSYSTEM_UPDATE")
-    EJ3 = descriptors.WatchedProperty("QUANTUMSYSTEM_UPDATE")
-    ECJ1 = descriptors.WatchedProperty("QUANTUMSYSTEM_UPDATE")
-    ECJ2 = descriptors.WatchedProperty("QUANTUMSYSTEM_UPDATE")
-    ECJ3 = descriptors.WatchedProperty("QUANTUMSYSTEM_UPDATE")
-    ECg1 = descriptors.WatchedProperty("QUANTUMSYSTEM_UPDATE")
-    ECg2 = descriptors.WatchedProperty("QUANTUMSYSTEM_UPDATE")
-    ng1 = descriptors.WatchedProperty("QUANTUMSYSTEM_UPDATE")
-    ng2 = descriptors.WatchedProperty("QUANTUMSYSTEM_UPDATE")
-    flux = descriptors.WatchedProperty("QUANTUMSYSTEM_UPDATE")
-    ncut = descriptors.WatchedProperty("QUANTUMSYSTEM_UPDATE")
+    EJ1 = descriptors.WatchedProperty(float, "QUANTUMSYSTEM_UPDATE")
+    EJ2 = descriptors.WatchedProperty(float, "QUANTUMSYSTEM_UPDATE")
+    EJ3 = descriptors.WatchedProperty(float, "QUANTUMSYSTEM_UPDATE")
+    ECJ1 = descriptors.WatchedProperty(float, "QUANTUMSYSTEM_UPDATE")
+    ECJ2 = descriptors.WatchedProperty(float, "QUANTUMSYSTEM_UPDATE")
+    ECJ3 = descriptors.WatchedProperty(float, "QUANTUMSYSTEM_UPDATE")
+    ECg1 = descriptors.WatchedProperty(float, "QUANTUMSYSTEM_UPDATE")
+    ECg2 = descriptors.WatchedProperty(float, "QUANTUMSYSTEM_UPDATE")
+    ng1 = descriptors.WatchedProperty(float, "QUANTUMSYSTEM_UPDATE")
+    ng2 = descriptors.WatchedProperty(float, "QUANTUMSYSTEM_UPDATE")
+    flux = descriptors.WatchedProperty(float, "QUANTUMSYSTEM_UPDATE")
+    ncut = descriptors.WatchedProperty(int, "QUANTUMSYSTEM_UPDATE")
 
     def __init__(
         self,
@@ -343,7 +346,6 @@ class FluxQubit(base.QubitBaseClass, serializers.Serializable, NoisyFluxQubit):
         self.flux = flux
         self.ncut = ncut
         self.truncated_dim = truncated_dim
-        self._evec_dtype = np.complex_
         self._default_grid = discretization.Grid1d(
             -np.pi / 2, 3 * np.pi / 2, 100
         )  # for plotting in phi_j basis
@@ -634,7 +636,7 @@ class FluxQubit(base.QubitBaseClass, serializers.Serializable, NoisyFluxQubit):
         """
         evals_count = max(which + 1, 3)
         if esys is None:
-            _, evecs = self.eigensys(evals_count)
+            _, evecs = self.eigensys(evals_count=evals_count)
         else:
             _, evecs = esys
         phi_grid = phi_grid or self._default_grid
