@@ -1,6 +1,7 @@
 # namedslots_array.py
 #
-# This file is part of scqubits.
+# This file is part of scqubits: a Python package for superconducting qubits,
+# Quantum 5, 583 (2021). https://quantum-journal.org/papers/q-2021-11-17-583/
 #
 #    Copyright (c) 2019 and later, Jens Koch and Peter Groszkowski
 #    All rights reserved.
@@ -89,6 +90,20 @@ def idx_for_value(value: Union[int, float, complex], param_vals: ndarray) -> int
 def convert_to_std_npindex(
     index_tuple: ExtIndexTuple, parameters: "Parameters"
 ) -> NpIndexTuple:
+    """
+    converts name-based and value-based indexing to standard numpy indexing
+
+    Parameters
+    ----------
+    index_tuple:
+        the indexing object to be converted
+    parameters:
+        records the parameters associated with the indices
+
+    Returns
+    -------
+        standard numpy index tuple
+    """
     extindex_obj_tuple = tuple(
         ExtIndexObject(entry, parameters, slot=slot_index)
         for slot_index, entry in enumerate(index_tuple)
@@ -114,7 +129,9 @@ def process_ellipsis(
     -------
         Processed multi-index not containing any `...`
     """
-    new_multi_idx: List[NpIndexNoEllipsis] = [slice(None, None, None)] * array.ndim
+    new_multi_idx: List[NpIndexNoEllipsis] = [
+        slice(None, None, None)
+    ] * array.ndim  # type:ignore
     # Replace the slice(None, None, None) entries, starting from beginning until
     # Ellipsis is encountered
     slot = 0
@@ -213,11 +230,20 @@ class ExtIndexTupleObject:
 
     def _name_based_to_np_index_exp(self) -> NpIndexTuple:
         """Converts a name-based multi-index into a standard numpy index_exp."""
-        converted_multi_index = [slice(None, None, None)] * self.slot_count
+        converted_multi_index: List[NpIndex] = [
+            slice(None, None, None)
+        ] * self.slot_count
+
         for extindex_object in self.extindex_tuple:
             if extindex_object.type != "slice.name":
                 raise TypeError("If one index is name-based, all indices must be.")
-            slot_index = self._parameters.index_by_name[extindex_object.name]
+            assert (
+                extindex_object.name is not None
+            ), "Internal error in NamedSlotsNdarray: index missing `name` attribute!"
+            slot_index = self._parameters.index_by_name[
+                extindex_object.name
+            ]  # type:ignore
+            assert isinstance(slot_index, int), "Internal NamedSlotsNdarray error"
             converted_multi_index[slot_index] = extindex_object.std_idx_entry
         return tuple(converted_multi_index)
 
@@ -251,7 +277,7 @@ class Parameters:
 
     def __init__(
         self,
-        paramvals_by_name: Dict[str, Union[ndarray, Iterable]],
+        paramvals_by_name: Dict[str, ndarray],
         paramnames_list: Optional[List[str]] = None,
     ) -> None:
         if paramnames_list is not None:
@@ -299,6 +325,10 @@ class Parameters:
     def __iter__(self):
         return iter(self.paramvals_list)
 
+    def ndim(self):
+        # Alias to support numpy's ndim
+        return len(self.paramnames_list)
+
     @property
     def counts_by_name(self) -> Dict[str, int]:
         """Returns a dictionary specifying for each parameter name the number of
@@ -310,7 +340,8 @@ class Parameters:
 
     @property
     def ranges(self) -> List[Iterable]:
-        """Return a list of range objects suitable for looping over each parametr set"""
+        """Return a list of range objects suitable for looping over each parameter
+        set"""
         return [range(count) for count in self.counts]
 
     @property
@@ -347,15 +378,15 @@ class Parameters:
         """
         if fixed_values is not None:
             # need to reformat as array of single-entry arrays
-            fixed_values = [np.asarray(value) for value in fixed_values]
+            fixed_values_list = [np.asarray(value) for value in fixed_values]
         else:
-            fixed_values = [
+            fixed_values_list = [
                 np.asarray([self[name][0]]) for name in fixed_parametername_list
             ]
 
         reduced_paramvals_by_name = {name: self[name] for name in self.paramnames_list}
         for index, name in enumerate(fixed_parametername_list):
-            reduced_paramvals_by_name[name] = fixed_values[index]
+            reduced_paramvals_by_name[name] = fixed_values_list[index]
         return Parameters(reduced_paramvals_by_name)
 
     def create_sliced(
@@ -416,7 +447,7 @@ class NamedSlotsNdarray(np.ndarray, Serializable):
     This class implements multi-dimensional arrays, for which the leading M dimensions
     are each associated with a slot name and a corresponding array of slot
     values (float or complex or str). All standard slicing of the multi-dimensional
-    array with integer-valued indices is supported as usual, e.g.
+    array with integer-valued indices is supported as usual, e.g.::
 
         some_array[0, 3:-1, -4, ::2]
 
@@ -430,12 +461,12 @@ class NamedSlotsNdarray(np.ndarray, Serializable):
     element (as measured by the absolute value of the difference for numbers,
     and an exact match for str) in the set of slot values.
 
-    As an example, consider the situation of two named value sets
+    As an example, consider the situation of two named value sets::
 
         values_by_slotname = {'param1': np.asarray([-4.4, -0.1, 0.3, 10.0]),
                               'param2': np.asarray([0.1*1j, 3.0 - 4.0*1j, 25.0])}
 
-    Then, the following are examples of value-based slicing:
+    Then, the following are examples of value-based slicing::
 
         some_array[0.25, 0:2]                   -->     some_array[2, 0:2]
         some_array[-3.0, 0.0:(2.0 - 4.0*1j)]    -->     some_array[0, 0:1]
@@ -443,13 +474,13 @@ class NamedSlotsNdarray(np.ndarray, Serializable):
 
     (2) Name-based slicing
     Sometimes, it is convenient to refer to one of the slots
-    by its name rather than its position within the multiple sets. As an example, let
+    by its name rather than its position within the multiple sets. As an example, let::
 
         values_by_slotname = {'ng': np.asarray([-0.1, 0.0, 0.1, 0.2]),
                              'flux': np.asarray([-1.0, -0.5, 0.0, 0.5, 1.0])}
 
     If we are interested in the slice of `some_array` obtained by setting 'flux' to a
-    value or the value associated with a given index, we can now use:
+    value or the value associated with a given index, we can now use::
 
         some_array['flux':0.5]            -->    some_array[:, 1]
         some_array['flux'::2, 'ng':-1]    -->    some_array[-1, :2]
@@ -469,9 +500,9 @@ class NamedSlotsNdarray(np.ndarray, Serializable):
     parameters: Parameters
 
     def __new__(
-        cls, input_array: np.ndarray, values_by_name: Dict[str, Iterable]
+        cls, input_array: np.ndarray, values_by_name: Dict[str, ndarray]
     ) -> "NamedSlotsNdarray":
-        implied_shape = tuple(len(values) for name, values in values_by_name.items())
+        implied_shape = tuple(len(values) for values in values_by_name.values())
         if input_array.shape[0 : len(values_by_name)] != implied_shape:
             raise ValueError(
                 "Given input array {} with shape {} not compatible with "
@@ -479,9 +510,7 @@ class NamedSlotsNdarray(np.ndarray, Serializable):
                     input_array, input_array.shape, implied_shape, values_by_name
                 )
             )
-
         obj = np.asarray(input_array).view(cls)
-
         obj._parameters = Parameters(values_by_name)
         return obj
 
@@ -493,16 +522,16 @@ class NamedSlotsNdarray(np.ndarray, Serializable):
     def __getitem__(self, multi_index: ExtIndices) -> Any:
         """Overwrites the magic method for element selection and slicing to support
         extended string and value based slicing."""
-        multi_index = np.index_exp[multi_index]  # convert to standardized tuple form
+        multi_index_std = np.index_exp[multi_index]  # convert to standard tuple form
         try:
-            obj = super().__getitem__(multi_index)
+            obj = super().__getitem__(multi_index_std)
             # This attempt fails if multi-index is string- or value-based
         except (TypeError, IndexError):
-            multi_index = convert_to_std_npindex(multi_index, self._parameters)
-            obj = super().__getitem__(multi_index)
+            multi_index_std = convert_to_std_npindex(multi_index_std, self._parameters)
+            obj = super().__getitem__(multi_index_std)
 
-        if Ellipsis in multi_index:
-            multi_index = process_ellipsis(self, multi_index)
+        if Ellipsis in multi_index_std:
+            multi_index_std = process_ellipsis(self, multi_index_std)
         # If the resulting obj is a sliced view of the current array, then we must
         # adjust the internal Parameters instance accordingly
         if isinstance(obj, NamedSlotsNdarray):
@@ -510,10 +539,10 @@ class NamedSlotsNdarray(np.ndarray, Serializable):
             dummy_array = np.empty(shape=self._parameters.counts)
             # Check whether all parameters are getting fixed; if not, adjust
             # Parameters for the new object
-            if not isinstance(dummy_array[multi_index[:param_count]], float):
+            if not isinstance(dummy_array[multi_index_std[:param_count]], float):
                 # have not reduced to one element
                 obj._parameters = self._parameters.create_sliced(
-                    multi_index[:param_count]
+                    multi_index_std[:param_count]
                 )
             elif (
                 obj._parameters.paramvals_by_name == self._parameters.paramvals_by_name
