@@ -148,13 +148,15 @@ def check_sync_status(func: Callable) -> Callable:
     @functools.wraps(func)
     def wrapper(self, *args, **kwargs):
         if self._out_of_sync:
-            warnings.warn(
-                "[scqubits] Some system parameters have been changed and"
-                " generated spectrum data could be outdated, potentially leading to"
-                " incorrect results. Spectral data can be refreshed via"
-                " <HilbertSpace>.generate_lookup() or <ParameterSweep>.run()",
-                Warning,
-            )
+            with warnings.catch_warnings():
+                warnings.simplefilter("always")
+                warnings.warn(
+                    "[scqubits] Some system parameters have been changed and"
+                    " generated spectrum data could be outdated, potentially leading to"
+                    " incorrect results. Spectral data can be refreshed via"
+                    " <HilbertSpace>.generate_lookup() or <ParameterSweep>.run()",
+                    Warning,
+                )
         return func(self, *args, **kwargs)
 
     return wrapper
